require "minitar"
require "./package"

class PackedParser
  def parse(str)
    raise NotImplementedError
  end

  def dump_string(io, str)
    unless str.nil?
      raise unless str.instance_of? String
      io.write(str)
    end
    io.write("\0")
  end

  # the same as dump_string but no trailing NUL
  # used for data with predefined length
  def dump_binary(io, data, length)
    raise if data.length != length
    io.write(data)
  end

  def dump_i32(io, int)
    bin = [int].pack("L") # 32-bit unsigned, native (!!) endian
    dump_binary(io, bin, 4)
  end

  def dump_array_str(io, arr)
    if arr.nil?
      dump_i32(io, 0)
    else
      dump_i32(io, arr.size)
      arr.each { |s| dump_string(io, s) }
    end
  end

  # return string
  def dump(io, pkg, options)
    dump_string(io, pkg.filename)
    dump_string(io, pkg.name)
    dump_string(io, pkg.base)
    dump_string(io, pkg.version)
    dump_string(io, pkg.description)
    dump_array_str(io, pkg.groups)
    dump_i32(io, pkg.download_size)
    dump_i32(io, pkg.install_size)
    dump_binary(io, pkg.md5sum, 16) unless options[:skip_md5]
    dump_binary(io, pkg.sha256sum, 32)
    dump_string(io, pkg.pgpsig) unless options[:skip_pgp]
    dump_string(io, pkg.url)
    dump_array_str(io, pkg.license)
    dump_string(io, pkg.arch)
    dump_i32(io, pkg.builddate)
    dump_string(io, pkg.packager)
    dump_array_str(io, pkg.replaces)
    dump_array_str(io, pkg.conflicts)
    dump_array_str(io, pkg.provides)
    dump_array_str(io, pkg.depends)
    dump_array_str(io, pkg.optdepends)
    dump_array_str(io, pkg.makedepends)
    dump_array_str(io, pkg.checkdepends)
  end
end

class PackedStorage
  def initialize(filename, **options)
    @filename = filename
    @parser = PackedParser.new
    @options = options
  end

  # Returns object of type Database
  def load
    db = Database.new
    file = File.new(filename)

    while not file.eof?
      db.packages << @parser.parse(file)
    end

    file.close
    db
  end

  def store(db)
    w = File.new(@filename, "wb")
    db.packages.each do |pkg|
      @parser.dump(w, pkg, @options)
    end
    w.close
  end
end
